class GeppettoModelException(Exception): pass


class IllegalArgumentException(Exception): pass


class GeppettoExecutionException(Exception):
    pass


class GeppettoAccessException(Exception):
    pass


class GeppettoVisitingException(Exception): pass


class GeppettoInitializationException(Exception): pass


class ModelInterpreterException(Exception): pass

class GeppettoDataSourceException(Exception): pass